"""
    client funcargs for py.test / django. Based on django_pytest
"""

from django.conf import settings
from django.test.client import Client
from django.test.utils import setup_test_environment, teardown_test_environment
from django.core.management import call_command
from django.core import mail

import pytest

@pytest.fixture
def django_client(request):
    """
        Setup / destroy testing database.
        Additionally, set the MEDIA_ROOT to the TEST_MEDIA_ROOT and clean up
        afterwards. This gives a cleaner fixture, and avoids testimages ending up
        in the MEDIA_ROOT folder.

        Of course we need to make sure we don't clean up a real MEDIA_ROOT. So
        only clean up the explicit TEST_MEDIA_ROOT, and only if it's different
        from the real MEDIA_ROOT
    """
    old_name = getattr(settings, 'DATABASE_NAME', 'default')

    def test_media_root():
        return getattr(settings, "TEST_MEDIA_ROOT", settings.MEDIA_ROOT)

    def setup():
        setup_test_environment()
        if not hasattr(settings, 'DEBUG'):
            settings.DEBUG = False
        from django.db import connection
        if 'south' in settings.INSTALLED_APPS:
            try:
                from south.management.commands import patch_for_test_db_setup
                patch_for_test_db_setup()
            except ImportError:
                pass
        connection.creation.create_test_db(verbosity=0, autoclobber=True)
        # call_command("migrate", database=connection.alias)
        c = Client()
        c.orig_media_root = settings.MEDIA_ROOT
        settings.MEDIA_ROOT = test_media_root()
        return c

    def teardown(client):
        teardown_test_environment()
        from django.db import connection
        connection.creation.destroy_test_db(old_name, verbosity=False)

    return request.cached_setup(setup, teardown, "session")

@pytest.fixture
def client(request):
    """ as django_client, but also flushes the database """
    def cleanup_media():
        return getattr(settings, "CLEANUP_MEDIA", False)

    def setup():
        return request.getfuncargvalue('django_client')

    def teardown(client):
        call_command('flush', verbosity=0, interactive=False)
        mail.outbox = []
        import shutil
        if cleanup_media() and settings.MEDIA_ROOT != client.orig_media_root:
            try:
                shutil.rmtree(settings.MEDIA_ROOT, ignore_errors=True)
            except OSError:
                pass

    return request.cached_setup(setup, teardown, "function")





class TestSessionStore(dict):
    """
        This will allow attributes to be set; e.g. .modified
    """

def create_request(method, *a, **b):
    """ setup a request with an anonymous user """
    from django.contrib.auth.models import AnonymousUser
    from django.test.client import RequestFactory

    if method == "GET":
        request = RequestFactory().get(*a, **b)
    elif method == "POST":
        request = RequestFactory().post(*a, **b)
    elif method == "PUT":
        request = RequestFactory().put(*a, **b)
    elif method == "DELETE":
        request = RequestFactory().delete(*a, **b)
    request.user = AnonymousUser()
    request.session = TestSessionStore()
    return request



from django.core.management.base import BaseCommand
import pytest
import optparse

from django.conf import settings

class Command(BaseCommand):
    option_list = BaseCommand.option_list + (
        optparse.make_option('-s',
            action='store_false',
            dest='capture',
            default=True,
            help='Capture output'),
        optparse.make_option('-k',
            dest='keyword',
            default='',
            help='Limit by keyword'),
        optparse.make_option('-p',
            action='store_false',
            dest='paste',
            default=True,
            help='Paste result to pastebin'),
        optparse.make_option('-f',
            action='store_true',
            dest='functional',
            default=False,
            help='Include functional tests'),
        optparse.make_option('-c',
            dest='coverage',
            default="",
            help='Perform coverage analysis, restrict to modules'),
        )

    def handle(self, *a, **b):
        ## http://pytest.org/latest/usage.html
        ## --tb=native is also usable
        coverage = False
        opts = ["--tb=short"]

        keyword_specified = False

        settings.DATABASES = dict(default=settings.TEST_DB)
        if not b.get('capture'):
            opts.append("-s")
        if b.get('keyword') and b['keyword']:
            opts.append("-k")
            opts.append(b['keyword'])
            keyword_specified = True

        if not b.get('paste'):
            opts.append("--pastebin=all")
        if not b.get('functional') and not keyword_specified:
            opts.append("-k")
            opts.append("-functional_")

        if b.get('coverage'):
            coverage = True
            coverage_modules = b.get('coverage', '').split(",")

        if coverage:
            import coverage
            conf = getattr(settings, 'COVERAGE_CONF', '')
            cov = coverage.coverage(source=coverage_modules, config_file=conf)
            cov.start()
        pytest.main(opts + list(a))
        if coverage:
            cov.stop()
            cov.report()





"""
Source: http://datadesk.latimes.com/posts/2012/06/test-your-django-app-with-travisci/
Slightly modified by Nicolas Kuttler.
"""

import os
import sys

from django.conf import settings


class QuickDjangoTest(object):
    """
    A quick way to run the Django test suite without a fully-configured project.

    Example usage:

        >>> QuickDjangoTest('app1', 'app2')

    Based on a script published by Lukasz Dziedzia at:
    http://stackoverflow.com/questions/3841725/how-to-launch-tests-for-django-reusable-app
    """
    DIRNAME = os.path.dirname(__file__)
    INSTALLED_APPS = (
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.admin',
    )
    WEBMASTER_VERIFICATION = {}

    def __init__(self, apps, installed_apps=(), *args, **kwargs):
        self.test_apps = apps
        self.installed_apps = installed_apps
        self.settings = kwargs
        self._tests()

    def _tests(self):
        """
        Fire up the Django test suite developed for version 1.2
        """
        default_settings = dict(
            DEBUG = True,
            DATABASES = {
                'default': {
                    'ENGINE': 'django.db.backends.sqlite3',
                    'NAME': 'memory:///',
                }
            },
            INSTALLED_APPS = self.INSTALLED_APPS + tuple(self.installed_apps),
            TEMPLATE_DIRS = (
                # './test_project/templates/',
            ),
            STATIC_URL='/static',
            SITE_ID=1,
            ROOT_URLCONF="twotest.emptyurls",
        )
        default_settings.update(self.settings)
        settings.configure(
            **default_settings
        )
        import pytest
        for app in self.test_apps:
            failures = pytest.main(["--tb=short", app])
            if failures:
                sys.exit(failures)


if __name__ == '__main__':
    """
    What do when the user hits this file from the shell.

    Example usage:

        $ python quicktest.py app1 app2

    """
    import argparse
    parser = argparse.ArgumentParser(
        usage="[args]",
        description="Run Django tests on the provided applications."
    )
    parser.add_argument('apps', nargs='+', type=str)
    options = parser.parse_args()
    QuickDjangoTest(options.apps)
